---
---
<!DOCTYPE html>
<html>
  <head>
    <title>libxml++ -- a C++ wrapper for the libxml2 XML parser library.</title>
    <meta http-equiv="Content-Type" content="text/html; charset=utf-8" />
    <link rel="stylesheet" href="style.css" type="text/css" />
  </head>

  <body>
    <div id="contents">
      <center><h1>{{page.title}}</h1></center>
      {{ content }}
    </div>
  </body>
</html>
